"use strict";
/*
 * Copyright 2018 Amazon.com, Inc. and its affiliates. All Rights Reserved.
 *
 * SPDX-License-Identifier: LicenseRef-.amazon.com.-AmznSL-1.0
 *
 * Licensed under the Amazon Software License (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *   http://aws.amazon.com/asl/
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExperimentOutlineProvider = void 0;
const vscode = require("vscode");
const sfb_f_1 = require("@alexa-games/sfb-f");
class ExperimentOutlineProvider {
    constructor(context) {
        this.context = context;
        this._onDidChangeTreeData = new vscode.EventEmitter();
        this.onDidChangeTreeData = this._onDidChangeTreeData.event;
        this.tree = {};
        vscode.window.onDidChangeActiveTextEditor(() => this.onActiveEditorChanged());
        vscode.workspace.onDidChangeTextDocument(e => this.onDocumentChanged(e));
    }
    async refresh(offset) {
        const enabled = vscode.window.activeTextEditor.document.languageId === 'abc-format';
        if (enabled) {
            let titleChanged = false;
            let names = {};
            if (this.tree) {
                for (let index of Object.keys(this.tree)) {
                    names[this.tree[index].id] = true;
                }
            }
            let sceneRegex = /^[\s]*?@.+?$/gm;
            let matches = this.editor.document.getText().match(sceneRegex);
            for (let newTitle of matches) {
                if (!names[newTitle.replace(/[\@]+/g, "").trim().toLowerCase()]) {
                    titleChanged = true;
                }
            }
            if (!this.tree || matches.length != this.tree.length || titleChanged) {
                await this.parseTree();
            }
            if (offset) {
                this._onDidChangeTreeData.fire(offset);
            }
            else {
                this._onDidChangeTreeData.fire(null);
            }
        }
    }
    async onActiveEditorChanged() {
        if (vscode.window.activeTextEditor) {
            if (vscode.window.activeTextEditor.document.uri.scheme === 'file') {
                const enabled = vscode.window.activeTextEditor.document.languageId === 'abc-format';
                vscode.commands.executeCommand('setContext', 'abcExperimentOutlineEnabled', enabled);
                if (enabled) {
                    await this.parseTree();
                    await this.refresh();
                }
            }
        }
        else {
            vscode.commands.executeCommand('setContext', 'abcExperimentOutlineEnabled', false);
        }
    }
    async onDocumentChanged(changeEvent) {
        console.log("Tree on Document Changed start.");
        if (this.editor && changeEvent.document.uri.toString() === this.editor.document.uri.toString()) {
            let titleChanged = false;
            let names = {};
            if (this.tree) {
                for (let index of Object.keys(this.tree)) {
                    names[this.tree[index].id] = true;
                }
            }
            let sceneRegex = /(?:^[\s]*?@).+?$/gm;
            let matches = this.editor.document.getText().match(sceneRegex);
            if (matches == null) {
                this.tree = [];
                this._onDidChangeTreeData.fire(null);
                console.log("Tree on Document Changed end.");
                return false;
            }
            else {
                for (let newTitle of matches) {
                    if (!names[newTitle.replace(/[\@]+/g, "").trim().toLowerCase()]) {
                        titleChanged = true;
                    }
                }
                if (!this.tree || matches.length != this.tree.length || titleChanged) {
                    await this.parseTree();
                    this._onDidChangeTreeData.fire(null);
                    console.log("Tree on Document Changed end.");
                    return true;
                }
            }
        }
        return null;
    }
    async parseTree() {
        const enabled = vscode.window.activeTextEditor.document.languageId === 'abc-format';
        if (enabled) {
            this.text = '';
            this.editor = vscode.window.activeTextEditor;
            this.tree = [];
            if (this.editor && this.editor.document) {
                this.text = this.editor.document.getText();
                let abcImporter = new sfb_f_1.SFBImporter();
                try {
                    let importedStory = await abcImporter.importABCStory("default", "", "", "", false, {
                        content: this.text,
                        ignoreSyntaxError: true
                    });
                    this.tree = importedStory.scenes;
                }
                catch (err) {
                    // ignore import error
                }
            }
        }
    }
    getChildren(scene) {
        if (scene) {
            return Promise.resolve([]);
        }
        else {
            return Promise.resolve(this.tree ? this.tree : []);
        }
    }
    getTreeItem(scene) {
        if (scene.id) {
            let treeItem = new vscode.TreeItem(scene.id, vscode.TreeItemCollapsibleState.None);
            let sourceLocation = this.editor.document.positionAt(scene.customProperties.sourceLocation);
            let letterAtPosition = this.editor.document.getText(new vscode.Range(sourceLocation, sourceLocation.translate(0, 1)));
            while (letterAtPosition !== "@" && this.editor.document.validatePosition(sourceLocation).isEqual(sourceLocation)) {
                sourceLocation = sourceLocation.translate(0, 1);
                letterAtPosition = this.editor.document.getText(new vscode.Range(sourceLocation, sourceLocation.translate(0, 1)));
            }
            treeItem.command = {
                command: 'extension.openStorySelectionExperiment',
                title: '',
                arguments: [new vscode.Range(sourceLocation, sourceLocation)]
            };
            treeItem.iconPath = {
                light: this.context.asAbsolutePath('resources/scene_icon.svg'),
                dark: this.context.asAbsolutePath('resources/scene_icon_dark.svg')
            };
            treeItem.contextValue = scene.type;
            return treeItem;
        }
        return null;
    }
    select(range) {
        this.editor.revealRange(range, vscode.TextEditorRevealType.AtTop);
    }
}
exports.ExperimentOutlineProvider = ExperimentOutlineProvider;
//# sourceMappingURL=ExperimentOutlineProvider.js.map